<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 2003-2004 robert lemke medienprojekte (rl@robertlemke.de)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is 
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/** 
 * Plugin 'Language Detection' for the 'rlmp_language_detection' extension.
 *
 * @author	robert lemke medienprojekte <rl@robertlemke.de>
 */


require_once(PATH_tslib.'class.tslib_pibase.php');

class tx_rlmplanguagedetection_pi1 extends tslib_pibase {
	var $prefixId = 'tx_rlmplanguagedetection_pi1';						// Same as class name
	var $scriptRelPath = 'pi1/class.tx_rlmplanguagedetection_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey = 'rlmp_language_detection';							// The extension key.
	var $conf = array();
	
	/**
	 * The constructor recognizes the browser's preferred languages and sets
	 * the reloads the page accordingly.
	 * 
	 * @param	string		$content: HTML content
	 * @param	array		$conf: The mandatory configuration array
	 * @return	void		
	 */
	function main($content,$conf)	{
		global $TSFE;		
		$this->conf = $conf;

		$acceptedLanguages = $this->getAcceptedLanguages();
		$availableLanguages = $this->conf['useOneTreeMethod'] ? $this->getSysLanguages() : $this->getMultipleTreeLanguages();
		$L = -1;
		while (count ($acceptedLanguages)>0) {
			$currentLanguage = array_shift ($acceptedLanguages);
			if ($availableLanguages[$currentLanguage]) {
				$L = $availableLanguages[$currentLanguage];
				break;
			}
		}
		
			// If there is no language set yet (no global parameter 'L') we 
			// reload the current page with the appropriate language
		if (!(t3lib_div::GPvar ('L')) && t3lib_div::GPvar('L')!='0' && $L>=0) {

			if ($this->conf['useOneTreeMethod']) {
				$page = $TSFE->page;
			} else {
				$sys_page = t3lib_div::makeInstance('t3lib_pageSelect');
				$sys_page->init(0);
				$page = $sys_page->getPage ($L);
			}
			$linkData = $TSFE->tmpl->linkData($page,'',0,'');			
			if (!stristr($linkData['totalURL'],'?')) { 
				$linkData['totalURL'].='?'; 
			}
			$locationURL = $this->conf['dontAddSchemeToURL'] ? $linkData['totalURL'] : t3lib_div::locationHeaderUrl($linkData['totalURL']);
			header('Location: '.$locationURL.'&L='.$L);
		}
	
	}

	/**
	 * Returns the preferred languages ("accepted languages") from the visitor's
	 * browser settings.
	 * 
	 * The accepted languages are described in RFC 2616.
	 * It's a list of language codes (e.g. 'en' for english), separated by
	 * comma (,). Each language may have a quality-value (e.g. 'q=0.7') which
	 * defines a priority. If no q-value is given, '1' is assumed. The q-value
	 * is separated from the language code by a semicolon (;) (e.g. 'de;q=0.7')
	 * 
	 * @return	array	An array containing the accepted languages; key = iso code, value = quality
	 */
	function getAcceptedLanguages () {
		$languages = array ();		
		$strAcceptedLanguage = t3lib_div::trimExplode (',',t3lib_div::getIndpEnv('HTTP_ACCEPT_LANGUAGE'),1);
		foreach ($strAcceptedLanguage as $languageLine) {
			list ($languageCode, $quality) = t3lib_div::trimExplode (';',$languageLine);
			$arrAcceptedLanguages[$languageCode] = $quality ? substr ($quality,2) : 1;
		}

			// Now sort the accepted languages by their quality and create an array containing only the language codes in the correct order.
		if (is_array ($arrAcceptedLanguages)) {
			arsort ($arrAcceptedLanguages);
			$languageCodes = array_keys ($arrAcceptedLanguages);
			if (is_array($languageCodes)) {
				reset ($languageCodes);
				while (list ($languageCode,$quality) = each ($languageCodes)){
					$quality = substr ($quality,0,2);
					$languages[$languageCode]=$quality;
				}
			}
		}
		return $languages;
	}

	/**
	 * Returns an array of sys_language records containing the ISO code as the key and the record's uid as the value
	 * 
	 * @return	array	sys_language records: ISO code => uid of sys_language record
	 * @access	private
	 */
	function getSysLanguages () {
			// Two options: prior TYPO3 3.6.0 the title of the sys_language entry must be one of the two-letter iso codes in order
			// to detect the language. But if the static_languages is installed and the sys_language record points to the correct
			// language, we can use that information instead.
		$tA = 'sys_language';
		$tB = 'static_languages';
		if (mysql(TYPO3_db, "SELECT lg_iso_2 FROM $tB LIMIT 0")) {
				// Table and field exist so create query for the new approach:
			$query  = "SELECT $tA.uid, $tB.lg_iso_2 as isocode FROM $tA LEFT JOIN $tB ON $tA.static_lang_isocode=$tB.uid WHERE 1";
			$query .= $this->cObj->enableFields ($tA);
			$query .= $this->cObj->enableFields ($tB);
		} else {
			$query = "SELECT $tA.uid, $tA.title as isocode FROM $tA WHERE 1 ".$this->cObj->enableFields($tA);
		}

		$res = mysql(TYPO3_db,$query);
		while ( $row = @mysql_fetch_assoc($res)) {
			$availableLanguages [trim(strtolower($row['isocode']))] = $row['uid'];
		}
		return $availableLanguages;
	}

	/**
	 * Returns an array of available languages defined in the TypoScript configuration for this plugin.
	 * Acts as an alternative for getSysLanguages ()
	 * 
	 * @return	array	available languages: ISO code => Page ID of languages' root page
	 * @access	private
	 */
	function getMultipleTreeLanguages () {
		foreach ($this->conf['multipleTreesRootPages.'] as $isoCode=>$uid) {
			$availableLanguages [trim(strtolower($isoCode))] = intval($uid);
		}
		return $availableLanguages;
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/rlmp_language_detection/pi1/class.tx_rlmplanguagedetection_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/rlmp_language_detection/pi1/class.tx_rlmplanguagedetection_pi1.php']);
}

?>